package uk.co.his.test.s24.ee.web;

import java.io.IOException;
import java.io.PrintWriter;
import java.util.ArrayList;
import java.util.List;
import java.util.logging.Logger;

import javax.naming.Context;
import javax.naming.InitialContext;
import javax.servlet.ServletException;
import javax.servlet.annotation.WebServlet;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.xml.ws.BindingProvider;
import javax.xml.ws.handler.Handler;

import org.jboss.jbossts.txbridge.outbound.JaxWSTxOutboundBridgeHandler;

import uk.co.his.test.ssb4.generated.SSB4;
import uk.co.his.test.ssb4.generated.SSB4Service;
import uk.co.his.test.ssb5.generated.SSB5;
import uk.co.his.test.ssb5.generated.SSB5Service;

import com.arjuna.mw.wst11.client.JaxWSHeaderContextProcessor;

/**
 * Servlet implementation class Test
 */
@WebServlet("/TestServlet")
public class TestServlet extends HttpServlet {
	private static final long serialVersionUID = 1L;
       
    /**
     * @see HttpServlet#HttpServlet()
     */
    public TestServlet() {
        super();
    }

	/**
	 * @see HttpServlet#doGet(HttpServletRequest request, HttpServletResponse response)
	 */
	protected void doGet(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {
		doPost(request, response);
	}

	/**
	 * @see HttpServlet#doPost(HttpServletRequest request, HttpServletResponse response)
	 */
	protected void doPost(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {
		

    	
		PrintWriter writer = new PrintWriter(response.getWriter());
    	SSB4 service1 = new SSB4Service().getSSB4Port();
    	SSB5 service2 = new SSB5Service().getSSB5Port();
    	//SSB4 service2 = new SSB4Service().getSSB4Port();
    	{
    	 BindingProvider bindingProvider = (BindingProvider) service1;
         List<Handler> handlers = new ArrayList<Handler>(1);
         handlers.add(new JaxWSTxOutboundBridgeHandler());
         handlers.add(new JaxWSHeaderContextProcessor());
         bindingProvider.getBinding().setHandlerChain(handlers);
    	}
    	{
    	 BindingProvider bindingProvider = (BindingProvider) service2;
         List<Handler> handlers = new ArrayList<Handler>(1);
         handlers.add(new JaxWSTxOutboundBridgeHandler());
         handlers.add(new JaxWSHeaderContextProcessor());
         bindingProvider.getBinding().setHandlerChain(handlers);
    	}
    	
		final String rowKey = "TEST";
    	final int increment = 1;
    	int expectedVal = increment;
    	javax.transaction.UserTransaction utx = null;
    	try {
    		Context initialContext = new InitialContext();
            utx = (javax.transaction.UserTransaction)initialContext.lookup("java:comp/UserTransaction");
            utx.begin();
            logger.info("Service1: " + service1.hello("Ping!"));
    		logger.info("Service2: " + service2.hello("Ping!"));
    		
    		logger.info("Add 100");
    		int bean1Val = service1.add(rowKey, 100);
    		logger.info("Added 100 to service1");
    		int bean2Val = service2.add(rowKey, 100);
    		logger.info("Added 100 to service2");
    		
    		logger.info("	Bean1: " + bean1Val);
    		logger.info("	Bean2: " + bean2Val);
    		assertTrue("Beans should be using different databases,  what one does should not effect the other", bean1Val==bean2Val);
    		logger.info("rollback");
    		utx.rollback();
    		
            utx.begin();
    		logger.info("Add 1");
    		bean1Val = service1.add(rowKey, increment);
    		bean2Val = service2.add(rowKey, increment);
    		logger.info("	Bean1: " + bean1Val);
    		logger.info("	Bean2: " + bean2Val);
    		assertTrue("Beans should be using different databases,  what one does should not effect the other", bean1Val==bean2Val);
    		assertTrue("Previous Txn rolledback, new value should be 1", bean1Val==expectedVal);
    		assertTrue("Previous Txn rolledback, new value should be 1", bean2Val==expectedVal);
    		logger.info("commit");
            utx.commit();
            expectedVal += increment;
            
            utx.begin();
    		logger.info("Add 1");
    		bean1Val = service1.add(rowKey, increment);
    		bean2Val = service2.add(rowKey, increment);
    		logger.info("	Bean1: " + bean1Val);
    		logger.info("	Bean2: " + bean2Val);
    		assertTrue("Beans should be using different databases,  what one does should not effect the other", bean1Val==bean2Val);
    		assertTrue("Previous Txn rolledback, new value should be 1", bean1Val==expectedVal);
    		assertTrue("Previous Txn rolledback, new value should be 1", bean2Val==expectedVal);
    		logger.info("rollback");
            utx.rollback();
            //Rolled back increment dioesn't happen
            //expectedVal += increment;
            
            utx.begin();
    		logger.info("Add 1");
    		bean1Val = service1.add(rowKey, increment);
    		bean2Val = service2.add(rowKey, increment);
    		logger.info("	Bean1: " + bean1Val);
    		logger.info("	Bean2: " + bean2Val);
    		assertTrue("Beans should be using different databases,  what one does should not effect the other", bean1Val==bean2Val);
    		assertTrue("Previous Txn rolledback, new value should be 1", bean1Val==expectedVal);
    		assertTrue("Previous Txn rolledback, new value should be 1", bean2Val==expectedVal);
    		logger.info("commit");
            utx.commit();
            expectedVal += increment;
            
            utx.begin();
    		logger.info("Add 1");
    		bean1Val = service1.add(rowKey, increment);
    		bean2Val = service2.add(rowKey, increment);
    		logger.info("	Bean1: " + bean1Val);
    		logger.info("	Bean2: " + bean2Val);
    		assertTrue("Beans should be using different databases,  what one does should not effect the other", bean1Val==bean2Val);
    		assertTrue("Previous Txn rolledback, new value should be 1", bean1Val==expectedVal);
    		assertTrue("Previous Txn rolledback, new value should be 1", bean2Val==expectedVal);
    		logger.info("commit");
            utx.commit();

		} catch (Exception e) {
			try {
				if(utx!=null)utx.rollback();
			} catch (Exception e1) {
				e1.printStackTrace();
			} 
			e.printStackTrace();
		} 
		writer.println("<html><title>Hello</title><H1>hello</H1><Body>Hello there</Body></html>");
		writer.flush();
	}

	
	private void assertTrue(String msg, boolean conditionMet)
	{
		if(!conditionMet)
		{
			logger.severe(msg);
		}
	}
	
	private static final Logger logger = Logger.getLogger(TestServlet.class
			.getName());
}
